@@ -276,7 +276,7 @@ revalidate_size (GtkSourceGutter *gutter)
 		gtk_text_view_set_border_window_size (GTK_TEXT_VIEW (gutter->priv->view),
 		                                      gutter->priv->window_type,
 		                                      1);
-		gutter->priv->size = -1;
+		do_redraw (gutter);
 	}
 	else if (window && !gutter->priv->renderers)
 	{
@@ -377,7 +377,7 @@ gtk_source_gutter_class_init (GtkSourceGutterClass *klass)
 			      3,
 			      GTK_TYPE_CELL_RENDERER,
 			      GTK_TYPE_TEXT_ITER,
-			      GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
+			      G_TYPE_POINTER);
 
 	/**
 	 * GtkSourceGutter::query-tooltip:
@@ -859,7 +859,6 @@ on_view_expose_event (GtkSourceView   *view,
 	size = calculate_sizes (gutter, sizes);
 	if (gutter->priv->size != size)
 	{
-
 		gint border_size;
 
 		border_size = gtk_text_view_get_border_window_size (text_view, gutter->priv->window_type);
